@@ -65,6 +65,77 @@ class SqliteDocumentPersistence(object):
         self.cursor.execute("pragma temp_store = memory")
         self.cursor.execute("pragma mmap_size = 30000000000")
 
+    def update_node(self, node):
+        self.cursor.execute('update cn set idx=?, pid=? where id=?',
+                            [node.index, node._parent_uuid,
+                             node.uuid])
+
+    def get_content_nodes(self, node_type, parent_node: ContentNode, include_children):
+        nodes = []
+
+        results = []
+        if include_children:
+
+            if node_type == "*":
+                query = """
+                            with recursive
+                            parent_node(id, pid, nt, idx) AS (
+                                VALUES (?,?,?,?)
+                                UNION ALL
+                                SELECT cns.id, cns.pid, cns.nt, cns.idx 
+                                FROM cn cns, parent_node
+                                WHERE parent_node.id = cns.pid  
+                            )
+                            SELECT id, pid, nt, idx from parent_node order by parent_node.idx, idx
+                            """
+
+                try:
+                    results = self.cursor.execute(query,
+                                                  [parent_node.uuid,
+                                                   parent_node.get_parent().uuid if parent_node.get_parent() else None,
+                                                   next(key for key, value in self.node_types.items() if
+                                                        value == parent_node.get_node_type()),
+                                                   parent_node.index]).fetchall()
+                except StopIteration:
+                    return []
+            else:
+                query = """
+                                with recursive
+                                parent_node(id, pid, nt, idx) AS (
+                                    VALUES (?,?,?,?)
+                                    UNION ALL
+                                    SELECT cns.id, cns.pid, cns.nt, cns.idx 
+                                    FROM cn cns, parent_node
+                                    WHERE parent_node.id = cns.pid  
+                                )
+                                SELECT id, pid, nt, idx from parent_node where nt=? order by parent_node.idx, idx
+                                """
+
+                try:
+                    results = self.cursor.execute(query,
+                                                  [parent_node.uuid,
+                                                   parent_node.get_parent().uuid if parent_node.get_parent() else None,
+                                                   next(key for key, value in self.node_types.items() if
+                                                        value == parent_node.get_node_type()),
+                                                   parent_node.index,
+                                                   next(key for key, value in self.node_types.items() if
+                                                        value == node_type)]).fetchall()
+                except StopIteration:
+                    return []
+        else:
+            query = "select id, pid, nt, idx from cn where pid=? and nt=? order by idx"
+            try:
+                results = self.cursor.execute(query,
+                                              [parent_node.uuid, next(key for key, value in self.node_types.items() if
+                                                                      value == node_type)]).fetchall()
+            except StopIteration:
+                return []
+
+        for raw_node in list(results):
+            nodes.append(self.__build_node(raw_node))
+
+        return nodes
+
     def initialize(self):
         if self.is_new:
             self.__build_db()
@@ -327,8 +398,7 @@ class SqliteDocumentPersistence(object):
         if self.document.content_node:
             self.__insert_node(self.document.content_node, None)
 
-    def get_bytes(self):
-
+    def sync(self):
         self.__update_metadata()
         self.cursor.execute("pragma optimize")
         self.connection.commit()
@@ -341,6 +411,8 @@ class SqliteDocumentPersistence(object):
         self.cursor.execute("pragma temp_store = memory")
         self.cursor.execute("pragma mmap_size = 30000000000")
 
+    def get_bytes(self):
+        self.sync()
         with open(self.current_filename, 'rb') as f:
             return f.read()
 
@@ -371,7 +443,7 @@ class SqliteDocumentPersistence(object):
         def get_all_node_ids(node):
             all_node_ids = []
             if not node.virtual:
-                all_node_ids.append([node.uuid])
+                all_node_ids.extend([node.uuid])
                 for child in node.get_children():
                     all_node_ids.extend(get_all_node_ids(child))
 
@@ -463,17 +535,23 @@ class PersistenceManager(object):
 
         self.node_cache.next_id = self._underlying_persistence.get_next_node_id()
 
+    def get_parent(self, node):
+        if node.uuid in self.node_parent_cache:
+            return self.node_cache.get_obj(self.node_parent_cache[node.uuid])
+        else:
+            return self._underlying_persistence.get_parent(node)
+
     def close(self):
         self._underlying_persistence.close()
 
-    def get_bytes(self):
-
+    def flush_cache(self):
         all_node_ids = []
         all_nodes = []
         all_content_parts = []
         all_features = []
         node_id_with_features = []
 
+
         logger.info("Merging cache to persistance")
         dirty_nodes = self.node_cache.get_dirty_objs()
 
@@ -516,6 +594,14 @@ class PersistenceManager(object):
 
         logger.info(f"Writing {len(all_features)} features")
         self._underlying_persistence.cursor.executemany(FEATURE_INSERT, all_features)
+
+        self._underlying_persistence.sync()
+
+    def get_content_nodes(self, node_type, parent_node, include_children):
+        return self._underlying_persistence.get_content_nodes(node_type, parent_node, include_children)
+
+    def get_bytes(self):
+        self.flush_cache()
         return self._underlying_persistence.get_bytes()
 
     def update_metadata(self):
@@ -583,6 +669,7 @@ class PersistenceManager(object):
         if node.uuid in self.node_parent_cache:
             self.child_cache[self.node_parent_cache[node.uuid]].remove(node)
             self.child_id_cache[self.node_parent_cache[node.uuid]].remove(node.uuid)
+            del self.node_parent_cache[node.uuid]
 
         self.content_parts_cache.pop(node.uuid, None)
         self.feature_cache.pop(node.uuid, None)
@@ -608,6 +695,9 @@ class PersistenceManager(object):
 
         return self.child_cache[node.uuid]
 
+    def update_node(self, node):
+        self._underlying_persistence.update_node(node)
+
     def update_content_parts(self, node, content_parts):
         self.content_parts_cache[node.uuid] = content_parts
 
@@ -628,6 +718,7 @@ class PersistenceManager(object):
         features = self.get_features(node)
         new_features = [i for i in features if not (i.feature_type == feature_type and i.name == name)]
         self.feature_cache[node.uuid] = new_features
+        self.node_cache.add_obj(node)
 
     def get_features(self, node):
 
@@ -642,5 +733,6 @@ class PersistenceManager(object):
         if node.uuid not in self.feature_cache:
             features = self._underlying_persistence.get_features(node)
             self.feature_cache[node.uuid] = features
-
+            self.node_cache.add_obj(node)
+        self.node_cache.add_obj(node)
         self.feature_cache[node.uuid].append(feature)
